// A program to display a cylinder using shading
//
// Use 'c' to render a simple coloured surface (no lighting)
// Use 'f' to render a flat shaded surface
// Use 's' to render a smooth shaded surface
// Use 'e' to enable backface culling
// Use 'd' to disable backface culling
// Use 't' to toggle two sided lighting
// Use 'h" to toggle head light
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include <math.h>
#include "Trackball.h"

#include <iostream>
using namespace std;


const float Pi = 3.14159265358979323846264338327f;
const int windowWidth=400;
const int windowHeight=400;

#define NUM_SEGMENTS 16

typedef enum {COLOURED, FLAT_SHADED, SMOOTH_SHADED} RENDER_OPTION;
RENDER_OPTION current_render_option=COLOURED;

// colour of the cylinder (used if shading disabled)
GLfloat colour[4];
// material properties of the cylinder (used if shading enabled)
GLfloat mat_specular[4];
GLfloat mat_ambient_and_diffuse[4];
GLfloat mat_shininess[1];

// Light properties
GLfloat light_ambient[4];
GLfloat light_diffuse[4];
GLfloat light_specular[4];
GLfloat light_positionScene[4];
GLfloat light_positionHeadlight[4];

CTrackball trackball;

bool twoSidedLighting=false;
bool headLightEnabled=false;


void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'c': current_render_option=COLOURED; 
					glDisable(GL_LIGHTING);
					glutPostRedisplay(); break;
		case 'f': current_render_option=FLAT_SHADED; 
					glEnable(GL_LIGHTING);
					glShadeModel(GL_FLAT);
					glutPostRedisplay(); break;
		case 's': current_render_option=SMOOTH_SHADED; 
					glEnable(GL_LIGHTING);
					glShadeModel(GL_SMOOTH);
					glutPostRedisplay(); break;
		case 'e': glCullFace(GL_BACK);
					glEnable(GL_CULL_FACE);
					glutPostRedisplay();  break;
		case 'd': glDisable(GL_CULL_FACE);
					glutPostRedisplay();  break;
		case 't': twoSidedLighting=!twoSidedLighting;
					glutPostRedisplay();  break;
		case 'h': headLightEnabled=!headLightEnabled;
cout << "\n headLightEnabled=" << headLightEnabled; cout.flush();
					glutPostRedisplay();  break;
		default: trackball.tbKeyboard(key);
	}
}

void display(void)
{
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
	if (headLightEnabled) glLightfv(GL_LIGHT0, GL_POSITION, light_positionHeadlight);
    gluLookAt(0,0,10, 0,0,0, 0,1,0); // camera is on the z-axis
	trackball.tbMatrix();			// rotate the cylinder using the trackball ...
	if (!headLightEnabled) glLightfv(GL_LIGHT0, GL_POSITION, light_positionScene);

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	float x,z,theta;
	
	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, (int) twoSidedLighting);

	glPushMatrix();
	glTranslatef(-1.5, 0,0);

	glBegin(GL_QUAD_STRIP);
	for (int segment=0; segment <= NUM_SEGMENTS; segment++)
	{
		theta=2.0f*Pi*(float) segment/(float) NUM_SEGMENTS;
		x = (float) cos(theta);		// x = r cos(theta), and r = 1
		z = (float) sin(theta);		// z = r sin(theta)
		if (current_render_option==SMOOTH_SHADED) glNormal3f(x,0,z);
		glVertex3f(x,-1,z);
		glVertex3f(x,1,z);
		if (current_render_option==FLAT_SHADED){
			theta=2.0f*Pi*((float) segment+0.5f)/(float) NUM_SEGMENTS;
			x = (float) cos(theta);		// x = r cos(theta), and r = 1
			z = (float) sin(theta);		// z = -r sin(theta)
			glNormal3f(x,0,z);}
	}
	glEnd();
	
	glTranslatef(3,0,0);
	glutSolidSphere(1.0, 32, 32);
	glPopMatrix();

	glFlush ();
	glutSwapBuffers();
}


void init(void) 
{
	// colour of the cylinder (used if shading disabled)
	colour[0]=1;						// red colour
	colour[1]=0;
	colour[2]=0;
	colour[3]=1;
	glColor4fv(colour);

	// material properties of the cylinder (used if shading enabled)
	mat_ambient_and_diffuse[0]=0.8;		// red material ...
	mat_ambient_and_diffuse[1]=0;
	mat_ambient_and_diffuse[2]=0;
	mat_ambient_and_diffuse[3]=1;
	mat_specular[0]=0.5f;				// ... with white highlights
	mat_specular[1]=0.5f;				// if light source is reflected
	mat_specular[2]=0.5f;				// on the material surface.
	mat_specular[3]=1;
	mat_shininess[0]=100;
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_ambient_and_diffuse);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	// light properties
	light_positionScene[0]=0.0f;
	light_positionScene[1]=0.0f;
	light_positionScene[2]=10.0f;
	light_positionScene[3]=1.0f;
	light_positionHeadlight[0]=0.0f;
	light_positionHeadlight[1]=0.0f;
	light_positionHeadlight[2]=0.0f;
	light_positionHeadlight[3]=1.0f;
	light_ambient[0]=0.4f;
	light_ambient[1]=0.4f;
	light_ambient[2]=0.4f;
	light_ambient[3]=1.0f;
	light_diffuse[0]=0.6f;
	light_diffuse[1]=0.6f;
	light_diffuse[2]=0.6f;
	light_diffuse[3]=1.0f;
	light_specular[0]=1.0f;
	light_specular[1]=1.0f;
	light_specular[2]=1.0f;
	light_specular[3]=1.0f;
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
	glLightfv(GL_LIGHT0, GL_POSITION, light_positionScene);
	glEnable(GL_LIGHT0);

	// select clearing color (for glClear)
	glClearColor (1,1,1,1);	// RGB-value for black
	// enable depth buffering
	glEnable(GL_DEPTH_TEST);

	trackball.tbInit(GLUT_LEFT_BUTTON);

	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(33,1,2,20);
}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a double buffered colour window
int main(int argc, char** argv)
{

	printf("\nUse 'c' to render a simple coloured surface (no lighting)");
	printf("\nUse 'f' to render a flat shaded surface");
	printf("\nUse 's' to render a smooth shaded surface");
	printf("\nUse 'e' to enable backface culling");
	printf("\nUse 'd' to disable backface culling");
	printf("\nUse 't' to toggle two sided lighting");
	printf("\nUse 'h' to toggle head light");
		
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Shaded Cylinder");
	init ();								// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
